# automatically generated by the FlatBuffers compiler, do not modify

# namespace: proto

class MessageType(object):
    NULL = 0
    HELLO = 1
    WELCOME = 2
    ABORT = 3
    CHALLENGE = 4
    AUTHENTICATE = 5
    GOODBYE = 6
    ERROR = 8
    PUBLISH = 16
    PUBLISHED = 17
    SUBSCRIBER_RECEIVED = 18
    SUBSCRIBE = 32
    SUBSCRIBED = 33
    UNSUBSCRIBE = 34
    UNSUBSCRIBED = 35
    EVENT = 36
    EVENT_RECEIVED = 37
    CALL = 48
    CANCEL = 49
    RESULT = 50
    REGISTER = 64
    REGISTERED = 65
    UNREGISTER = 66
    UNREGISTERED = 67
    INVOCATION = 68
    INTERRUPT = 69
    YIELD = 70

