# automatically generated by the FlatBuffers compiler, do not modify

# namespace: proto

import flatbuffers

class AuthScramWelcome(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsAuthScramWelcome(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = AuthScramWelcome()
        x.Init(buf, n + offset)
        return x

    # AuthScramWelcome
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # AuthScramWelcome
    def Verifier(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def AuthScramWelcomeStart(builder): builder.StartObject(1)
def AuthScramWelcomeAddVerifier(builder, verifier): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(verifier), 0)
def AuthScramWelcomeEnd(builder): return builder.EndObject()
